import 'dart:async';

import 'package:audioplayers/audioplayers.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:multimedia_gallery/extension/constants.dart';
import 'package:multimedia_gallery/extension/string_formatter.dart';
import 'package:multimedia_gallery/audio/widget/image_background_gradient_widget.dart';

/// AudioViewer class to be used to get the audio player ui and function.
class AudioViewer extends StatefulWidget {
  final ImageProvider? image;
  final String? songName;
  final String? artistName;
  final EdgeInsets? screenPadding;
  final double? imageWidth;
  final BorderRadius? imageBorder;
  final BoxFit? imageFit;
  final TextStyle? songNameTextStyle;
  final TextStyle? artistNameTextStyle;
  final SliderThemeData? sliderTheme;
  final Icon? icon;
  final ButtonStyle? iconStyle;
  final void Function()? onPressed;
  final Source path;

  const AudioViewer(
      {Key? key,
      required this.path,
      required this.image,
      required this.songName,
      required this.artistName,
      this.screenPadding,
      this.imageBorder,
      this.imageWidth,
      this.imageFit,
      this.artistNameTextStyle,
      this.songNameTextStyle,
      this.sliderTheme,
      this.icon,
      this.iconStyle,
      this.onPressed})
      : super(key: key);

  @override
  State<AudioViewer> createState() => _AudioViewerState();
}

class _AudioViewerState extends State<AudioViewer> {
  late ScrollController _titleController;
  late ScrollController _artistController;

  /// The audio player controller. This [initialize] the audio player as the controller
  /// to listen to the event of the audio player.
  AudioPlayer player = AudioPlayer();

  /// The timestamp of audio. Will change according to the current timestamp
  /// of the audio being played
  Duration? audioTimestamp;

  /// The duration of the audio.
  Duration? audioDuration;

  /// The state of the audio player. The state can be updated to
  /// play, paused or resumed.
  PlayerState? state;

  /// The position of the slider thumb at the given timestamp.
  /// Will be updated based on the current timestamp of the audio being played.
  double? sliderPosition;

  /// The maximum position of the slider. The value is based on the
  /// duration of the audio file.
  double? sliderMaxPosition;

  /// The duration listener. Used to watch or listen to the
  /// duration of the audio file.
  StreamSubscription? durationSubscription;

  /// The timestamp listener. Used to watch or listen to the current
  /// timestamp and the position of the slider.
  StreamSubscription? positionSubscription;

  /// The audio completion listener. Used to watch or listen to the
  /// activity after the audio completed played.
  StreamSubscription? playerCompleteSubscription;

  /// The audio state listener. Used to watch or listen to the
  /// activity while the audio is played.
  StreamSubscription? playerStateChangeSubscription;

  @override
  void initState() {
    super.initState();
    textController();
    getDuration();
    playAudio();
    getCurrentTimestamp();
    playerController();
  }

  @override
  void dispose() {
    _titleController.dispose();
    _artistController.dispose();
    durationSubscription?.cancel();
    positionSubscription?.cancel();
    playerCompleteSubscription?.cancel();
    playerStateChangeSubscription?.cancel();
    player.dispose();
    super.dispose();
  }

  void textController() {
    _titleController = ScrollController();
    _artistController = ScrollController();
    _startAutoScroll(_titleController);
    _startAutoScroll(_artistController);
  }
  /// The text auto scroller. This scroll controller is to automate
  /// the text scroll from left to right the text is overflowing.
  void _startAutoScroll(ScrollController controller) {
    Timer.periodic(const Duration(seconds: 10), (_) {
      if (controller.hasClients) {
        double maxScrollExtent = controller.position.maxScrollExtent;
        double minScrollExtent = controller.position.minScrollExtent;
        if (controller.offset != maxScrollExtent) {
          controller.animateTo(maxScrollExtent, duration: const Duration(seconds: 5), curve: Curves.linear);
        } else {
          controller.animateTo(minScrollExtent, duration: const Duration(seconds: 5), curve: Curves.linear);
        }
      }
    });
  }


  /// This method is to play the audio file on [initState].
  /// This method will get the audio file path and play the audio file if the audio file can be found.
  void playAudio(){
    player.play(widget.path);
  }

  /// This method is to set the state of the audio player to [play].
  /// This method only can be invoke if the audio file exists
  /// and the state of the audio player is pause.
  void resumeAudio() {
    player.resume();
  }

  /// This method is to set the state of the audio player to [pause].
  /// This method only can be invoke if the audio file exists
  /// and the state of the audio player is play.
  void pauseAudio() {
    player.pause();
  }

  /// This method is to set the duration of the audio file. This method is necessary to
  /// be invoke in the [initState] since [player.getDuration] is a future method.
  /// After the [player.getDuration] is invoked, a final parameter [audioDuration] is set to this value.
  void getDuration() {
    player.getDuration().then((value) => setState(() => audioDuration = value));
  }

  /// This method is to set the current timestamp that the audio file is played.
  /// This method is necessary to be invoked in [initState] since the player
  /// is a future method. After the [player.getCurrentPosition] is invoked, a
  /// final parameter [audioTimestamp] is set to this value.
  void getCurrentTimestamp() {
    player.getCurrentPosition().then((value) => setState(() => audioTimestamp = value));
  }

  /// The audio player controller. This controller method is to listen and update
  /// the parameter to be updated.
  void playerController() {
    /// The audio duration listener. This is to update the audio file duration when
    /// there are changes to the audio duration. This method can be used when changing
    /// audio file to play.
    durationSubscription = player.onDurationChanged.listen((d) {
      setState(() {
        audioDuration = d;
        sliderMaxPosition = d.inMilliseconds.toDouble();
      });
    });

    /// The current timestamp listener. This listener is assign to update
    /// current timestamp and the slider thumb position when the audio player is running.
    positionSubscription = player.onPositionChanged.listen(
      (p) => setState(() {
        audioTimestamp = p;
        sliderPosition = p.inMilliseconds.toDouble();
      }),
    );

    /// The audio onPlayerComplete listener. This listener is to update the parameter
    /// after the audio player has complete played.
    playerCompleteSubscription = player.onPlayerComplete.listen((event) {
      setState(() {
        state = PlayerState.stopped;
      });
    });

    /// The audio onPlayerComplete listener. This listener is to update the state
    /// of the audio player after the audio player has complete played.
    playerStateChangeSubscription = player.onPlayerStateChanged.listen((state) {
      setState(() {
        state = state;
      });
    });
  }

  /// The timestamp seeker. This method is to update the audio timestamp when the audio
  /// slider is slide to another timestamp.
  void seekTo(double value) {
    player.seek(Duration(milliseconds: value.round()));
  }

  @override
  Widget build(BuildContext context) {
    SystemChrome.setEnabledSystemUIMode(SystemUiMode.immersive);
    return widget.image.toString().contains('.gif')
        ? Container(
            decoration: BoxDecoration(
                image: DecorationImage(
                    image: widget.image ?? const AssetImage(''), fit: BoxFit.none, repeat: ImageRepeat.repeat)),
            child: Stack(children: [Container(color: Colors.black38), _buildContent()]))
        : GradientBackground(image: widget.image ?? const AssetImage(''), child: _buildContent());
  }


  Widget _buildContent() {
    return Scaffold(
        resizeToAvoidBottomInset: false,
        backgroundColor: Colors.transparent,
        appBar: AppBar(
            backgroundColor: Colors.transparent,
            leading: IconButton(
                icon: const Icon(Icons.arrow_back), onPressed: () => Navigator.pop(context), color: Colors.white)),
        body: SingleChildScrollView(
          child: Container(
              height: MediaQuery.of(context).size.height,
              padding: widget.screenPadding ?? const EdgeInsets.all(16),
              child: Column(mainAxisAlignment: MainAxisAlignment.center, children: [
                widget.image.toString().contains('.gif')
                    ? Expanded(flex: 2, child: SizedBox(height: MediaQuery.of(context).size.height / 1.8))
                    : Flexible(
                        flex: 2,
                        child: FittedBox(
                          child: ClipRRect(
                              borderRadius: BorderRadius.circular(10),
                              child: Card(
                                elevation: 4,
                                clipBehavior: Clip.antiAlias,
                                child: Image(
                                    image: widget.image ?? const AssetImage(''),
                                    width: MediaQuery.of(context).size.shortestSide / 2,
                                    fit: BoxFit.fill),
                              )),
                        ),
                      ),
                const SizedBox(height: 10),
                Expanded(
                  flex: 2,
                    child: Column(mainAxisSize: MainAxisSize.min, children: [
                      SingleChildScrollView(
                          scrollDirection: Axis.horizontal,
                          controller: _titleController,
                          child: Column(mainAxisSize: MainAxisSize.min, children: [
                            Flexible(
                                child: FittedBox(
                                    fit: BoxFit.scaleDown,
                                    child: Text(widget.songName ?? '',
                                        style: widget.songNameTextStyle ??
                                            const TextStyle(
                                                fontSize: 18, fontWeight: FontWeight.bold, color: Colors.white))))
                          ])),
                      SingleChildScrollView(
                          scrollDirection: Axis.horizontal,
                          controller: _artistController,
                          child: Column(mainAxisSize: MainAxisSize.min, children: [
                            Flexible(
                                child: FittedBox(
                                    fit: BoxFit.scaleDown,
                                    child: Text(widget.artistName ?? '',
                                        style: widget.artistNameTextStyle ??
                                            const TextStyle(
                                                fontSize: 18, fontWeight: FontWeight.w600, color: Colors.white70))))
                          ])),
                      p10,
                      SliderTheme(
                          data: widget.sliderTheme ??
                              SliderThemeData(
                                  overlayShape: SliderComponentShape.noOverlay,
                                  thumbColor: Colors.white,
                                  activeTrackColor: Colors.white,
                                  inactiveTrackColor: Colors.black26),
                          child: Slider(
                              value: sliderPosition ?? 0,
                              min: 0,
                              max: sliderMaxPosition ?? 0,
                              onChanged: (value) {
                                sliderPosition = value;
                              },
                              onChangeEnd: (value) {
                                seekTo(value);
                              })),
                      Padding(
                          padding: const EdgeInsets.all(10.0),
                          child: Row(mainAxisAlignment: MainAxisAlignment.spaceBetween, children: [
                            Text(formatDuration(audioTimestamp ?? Duration.zero),
                                style: const TextStyle(fontSize: 14, fontWeight: FontWeight.bold, color: Colors.white)),
                            Text(formatDuration((audioDuration ?? Duration.zero) - (audioTimestamp ?? Duration.zero)),
                                style: const TextStyle(fontSize: 14, fontWeight: FontWeight.bold, color: Colors.white))
                          ])),
                      IconButton.filled(
                          onPressed: widget.onPressed ??
                              () {
                                setState(() {
                                  (player.state.name == 'playing') ? pauseAudio() : resumeAudio();
                                });
                              },
                          icon: widget.icon ?? Icon(player.state.name == 'playing' ? Icons.pause : Icons.play_arrow),
                          style: widget.iconStyle ??
                              const ButtonStyle(
                                  backgroundColor: MaterialStatePropertyAll<Color>(Colors.white),
                                  iconColor: MaterialStatePropertyAll<Color>(Colors.black),
                                  iconSize: MaterialStatePropertyAll(20)))
                    ]))
              ])),
        ));
  }
}
